/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import picto.app.interfaces.Localizable;
import picto.utils.Config;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class ColorChooser extends JDialog implements Localizable, ActionListener{
    private static final long serialVersionUID = 165789468537246352L;

    /** Creates new form ColorChooser */
    public ColorChooser(Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        updateTexts();
        loadColors();
        addWeakReference();
    }

    private String title = _("Select Color");
    private String s = "<html><i><b>";
    private String e = "</b></i></html>";
    @Override public final void updateTexts() {
        title = _("Select Color");
        setTitle(title);
        closeButton.setText(_("Close"));
        resetButton.setText(_("Defaults"));
        if (Config.getColor("colorButtonBackgroundEmpty").equals(
                Config.getDefaultColor("colorButtonBackgroundEmpty"))){
            defaultCellLabel.setText(_("Default cell"));
        } else {
            defaultCellLabel.setText(s+_("Default cell")+e);
        }
        if (Config.getColor("colorButtonBackgroundSomething").equals(
                Config.getDefaultColor("colorButtonBackgroundSomething"))){
            markedCellLabel.setText(_("Marked cell"));
        } else {
            markedCellLabel.setText(s+_("Marked cell")+e);
        }
        if (Config.getColor("colorButtonBackgroundNothing").equals(
                Config.getDefaultColor("colorButtonBackgroundNothing"))){
            emptyCellLabel.setText(_("Empty cell"));
        } else {
            emptyCellLabel.setText(s+_("Empty cell")+e);
        }
        if (Config.getColor("colorButtonBorderColor").equals(
                Config.getDefaultColor("colorButtonBorderColor"))){
            innerBorderLabel.setText(_("Inner border"));
        } else {
            innerBorderLabel.setText(s+_("Inner border")+e);
        }
        if (Config.getColor("headerBackgroundColor").equals(
                Config.getDefaultColor("headerBackgroundColor"))){
            headersLabel.setText(_("Headers"));
        } else {
            headersLabel.setText(s+_("Headers")+e);
        }
        if (Config.getColor("headerBorderColor").equals(
                Config.getDefaultColor("headerBorderColor"))){
            headerBorderLabel.setText(_("Header border"));
        } else {
            headerBorderLabel.setText(s+_("Header border")+e);
        }
        if (Config.getColor("headerBackgroundFocusColor").equals(
                Config.getDefaultColor("headerBackgroundFocusColor"))){
            headerFocusLabel.setText(_("Header focus"));
        } else {
            headerFocusLabel.setText(s+_("Header focus")+e);
        }
        if (Config.getColor("backgroundColor").equals(
                Config.getDefaultColor("backgroundColor"))){
            backgroundColorLabel.setText(_("Background color"));
        } else {
            backgroundColorLabel.setText(s+_("Background color")+e);
        }
        if (Config.getColor("strikedTextForeground").equals(
                Config.getDefaultColor("strikedTextForeground"))){
            strikedHeaderColorLabel.setText(_("Striked header color"));
        } else {
            strikedHeaderColorLabel.setText(s+_("Striked header color")+e);
        }
        headerFontButton.setForeground(Config.getColor("headerFontForeground"));
        headerFontButton.setFont(Config.getFont("headerFont"));
        if (Config.getFont("headerFont").equals(
                Config.getDefaultFont("headerFont"))){
            headerFontButton.setText(_("Header font"));
        } else {
            headerFontButton.setText(s+_("Header font")+e);
        }
        ((MainWindow)getParent()).updateColors();
    }

    @Override public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    private void loadColors(){
        colorButtonBackgroundEmptyButton.setBackground(Config.getColor("colorButtonBackgroundEmpty"));
        colorButtonBackgroundSomethingButton.setBackground(Config.getColor("colorButtonBackgroundSomething"));
        colorButtonBackgroundNothingButton.setBackground(Config.getColor("colorButtonBackgroundNothing"));
        backgroundButton.setBackground(Config.getColor("colorButtonBorderColor"));
        headerBackgroundColoButton.setBackground(Config.getColor("headerBackgroundColor"));
        headerBorderColorButton.setBackground(Config.getColor("headerBorderColor"));
        headerBackgroundFocusColoButton.setBackground(Config.getColor("headerBackgroundFocusColor"));
        backgroundColorButton.setBackground(Config.getColor("backgroundColor"));
        strikedTextForegroundButton.setBackground(Config.getColor("strikedTextForeground"));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        closeButton = new javax.swing.JButton();
        resetButton = new javax.swing.JButton();
        backgroundColorButton = new javax.swing.JButton();
        colorButtonBackgroundSomethingButton = new javax.swing.JButton();
        backgroundColorLabel = new javax.swing.JLabel();
        colorButtonBackgroundNothingButton = new javax.swing.JButton();
        backgroundButton = new javax.swing.JButton();
        headerFocusLabel = new javax.swing.JLabel();
        headerBackgroundFocusColoButton = new javax.swing.JButton();
        headerBackgroundColoButton = new javax.swing.JButton();
        strikedTextForegroundButton = new javax.swing.JButton();
        headerBorderColorButton = new javax.swing.JButton();
        defaultCellLabel = new javax.swing.JLabel();
        headerFontButton = new javax.swing.JButton();
        markedCellLabel = new javax.swing.JLabel();
        strikedHeaderColorLabel = new javax.swing.JLabel();
        innerBorderLabel = new javax.swing.JLabel();
        colorButtonBackgroundEmptyButton = new javax.swing.JButton();
        emptyCellLabel = new javax.swing.JLabel();
        headerBorderLabel = new javax.swing.JLabel();
        headersLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setAlwaysOnTop(true);

        closeButton.setText("jButton1");
        closeButton.addActionListener(this);

        resetButton.setText("jButton2");
        resetButton.addActionListener(this);

        backgroundColorButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        backgroundColorButton.setContentAreaFilled(false);
        backgroundColorButton.setOpaque(true);
        backgroundColorButton.addActionListener(this);

        colorButtonBackgroundSomethingButton.setText(" ");
        colorButtonBackgroundSomethingButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        colorButtonBackgroundSomethingButton.setContentAreaFilled(false);
        colorButtonBackgroundSomethingButton.setOpaque(true);
        colorButtonBackgroundSomethingButton.addActionListener(this);

        backgroundColorLabel.setText("background color");

        colorButtonBackgroundNothingButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        colorButtonBackgroundNothingButton.setContentAreaFilled(false);
        colorButtonBackgroundNothingButton.setOpaque(true);
        colorButtonBackgroundNothingButton.addActionListener(this);

        backgroundButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        backgroundButton.setContentAreaFilled(false);
        backgroundButton.setOpaque(true);
        backgroundButton.addActionListener(this);

        headerFocusLabel.setText("background color");

        headerBackgroundFocusColoButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        headerBackgroundFocusColoButton.setContentAreaFilled(false);
        headerBackgroundFocusColoButton.setOpaque(true);
        headerBackgroundFocusColoButton.addActionListener(this);

        headerBackgroundColoButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        headerBackgroundColoButton.setContentAreaFilled(false);
        headerBackgroundColoButton.setOpaque(true);
        headerBackgroundColoButton.addActionListener(this);

        strikedTextForegroundButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        strikedTextForegroundButton.setContentAreaFilled(false);
        strikedTextForegroundButton.setOpaque(true);
        strikedTextForegroundButton.addActionListener(this);

        headerBorderColorButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        headerBorderColorButton.setContentAreaFilled(false);
        headerBorderColorButton.setOpaque(true);
        headerBorderColorButton.addActionListener(this);

        defaultCellLabel.setText("Empty field");

        headerFontButton.setText("Font");
        headerFontButton.addActionListener(this);

        markedCellLabel.setText("Full field");

        strikedHeaderColorLabel.setText("background color");

        innerBorderLabel.setText("Inner border");

        colorButtonBackgroundEmptyButton.setBackground(new java.awt.Color(255, 51, 0));
        colorButtonBackgroundEmptyButton.setText(" ");
        colorButtonBackgroundEmptyButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        colorButtonBackgroundEmptyButton.setContentAreaFilled(false);
        colorButtonBackgroundEmptyButton.setOpaque(true);
        colorButtonBackgroundEmptyButton.addActionListener(this);

        emptyCellLabel.setText("None field");

        headerBorderLabel.setText("header border");

        headersLabel.setText("header");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(headerFontButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(resetButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(closeButton))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(colorButtonBackgroundEmptyButton, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(defaultCellLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(colorButtonBackgroundSomethingButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(markedCellLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(colorButtonBackgroundNothingButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(emptyCellLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(backgroundButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(innerBorderLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(headerBackgroundColoButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(headersLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(headerBorderColorButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(headerBorderLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(headerBackgroundFocusColoButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(headerFocusLabel))
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(backgroundColorButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(backgroundColorLabel)))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(strikedTextForegroundButton)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(strikedHeaderColorLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 189, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(0, 0, Short.MAX_VALUE)))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {backgroundButton, backgroundColorButton, colorButtonBackgroundEmptyButton, colorButtonBackgroundNothingButton, colorButtonBackgroundSomethingButton, headerBackgroundColoButton, headerBackgroundFocusColoButton, headerBorderColorButton, strikedTextForegroundButton});

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {backgroundColorLabel, defaultCellLabel, emptyCellLabel, headerBorderLabel, headerFocusLabel, headersLabel, innerBorderLabel, markedCellLabel, strikedHeaderColorLabel});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(colorButtonBackgroundEmptyButton)
                    .addComponent(defaultCellLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(colorButtonBackgroundSomethingButton)
                    .addComponent(markedCellLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(colorButtonBackgroundNothingButton)
                    .addComponent(emptyCellLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(backgroundButton)
                    .addComponent(innerBorderLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(headerBackgroundColoButton)
                    .addComponent(headersLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(headerBorderColorButton)
                    .addComponent(headerBorderLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(headerBackgroundFocusColoButton)
                    .addComponent(headerFocusLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(backgroundColorButton)
                    .addComponent(backgroundColorLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(strikedTextForegroundButton)
                    .addComponent(strikedHeaderColorLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(headerFontButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(closeButton)
                    .addComponent(resetButton))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {backgroundButton, backgroundColorButton, backgroundColorLabel, colorButtonBackgroundEmptyButton, colorButtonBackgroundNothingButton, colorButtonBackgroundSomethingButton, defaultCellLabel, emptyCellLabel, headerBackgroundColoButton, headerBackgroundFocusColoButton, headerBorderColorButton, headerBorderLabel, headerFocusLabel, headersLabel, innerBorderLabel, markedCellLabel, strikedHeaderColorLabel, strikedTextForegroundButton});

        pack();
    }

    // Code for dispatching events from components to event handlers.

    public void actionPerformed(java.awt.event.ActionEvent evt) {
        if (evt.getSource() == colorButtonBackgroundEmptyButton) {
            ColorChooser.this.colorButtonBackgroundEmptyButtonActionPerformed(evt);
        }
        else if (evt.getSource() == colorButtonBackgroundSomethingButton) {
            ColorChooser.this.colorButtonBackgroundSomethingButtonActionPerformed(evt);
        }
        else if (evt.getSource() == colorButtonBackgroundNothingButton) {
            ColorChooser.this.colorButtonBackgroundNothingButtonActionPerformed(evt);
        }
        else if (evt.getSource() == backgroundButton) {
            ColorChooser.this.backgroundButtonActionPerformed(evt);
        }
        else if (evt.getSource() == headerBackgroundColoButton) {
            ColorChooser.this.headerBackgroundColoButtonActionPerformed(evt);
        }
        else if (evt.getSource() == headerBorderColorButton) {
            ColorChooser.this.headerBorderColorButtonActionPerformed(evt);
        }
        else if (evt.getSource() == headerBackgroundFocusColoButton) {
            ColorChooser.this.headerBackgroundFocusColoButtonActionPerformed(evt);
        }
        else if (evt.getSource() == backgroundColorButton) {
            ColorChooser.this.backgroundColorButtonActionPerformed(evt);
        }
        else if (evt.getSource() == headerFontButton) {
            ColorChooser.this.headerFontButtonActionPerformed(evt);
        }
        else if (evt.getSource() == strikedTextForegroundButton) {
            ColorChooser.this.strikedTextForegroundButtonActionPerformed(evt);
        }
        else if (evt.getSource() == closeButton) {
            ColorChooser.this.closeButtonActionPerformed(evt);
        }
        else if (evt.getSource() == resetButton) {
            ColorChooser.this.resetButtonActionPerformed(evt);
        }
    }// </editor-fold>//GEN-END:initComponents

    private void colorButtonBackgroundEmptyButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_colorButtonBackgroundEmptyButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("colorButtonBackgroundEmpty"));
        if (temp != null){
            Config.setColor("colorButtonBackgroundEmpty", temp);
            colorButtonBackgroundEmptyButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_colorButtonBackgroundEmptyButtonActionPerformed

    private void colorButtonBackgroundSomethingButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_colorButtonBackgroundSomethingButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("colorButtonBackgroundSomething"));
        if (temp != null){
            Config.setColor("colorButtonBackgroundSomething", temp);
            colorButtonBackgroundSomethingButton.setBackground(temp);
            PreviewWindow.getInstance().pp.setColor(temp);
            updateTexts();
        }
    }//GEN-LAST:event_colorButtonBackgroundSomethingButtonActionPerformed

    private void colorButtonBackgroundNothingButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_colorButtonBackgroundNothingButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("colorButtonBackgroundNothing"));
        if (temp != null){
            Config.setColor("colorButtonBackgroundNothing", temp);
            colorButtonBackgroundNothingButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_colorButtonBackgroundNothingButtonActionPerformed

    private void backgroundButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_backgroundButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("colorButtonBorderColor"));
        if (temp != null){
            Config.setColor("colorButtonBorderColor", temp);
            backgroundButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_backgroundButtonActionPerformed

    private void headerBackgroundColoButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_headerBackgroundColoButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("headerBackgroundColor"));
        if (temp != null){
            Config.setColor("headerBackgroundColor", temp);
            headerBackgroundColoButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_headerBackgroundColoButtonActionPerformed

    private void headerBorderColorButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_headerBorderColorButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("headerBorderColor"));
        if (temp != null){
            Config.setColor("headerBorderColor", temp);
            headerBorderColorButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_headerBorderColorButtonActionPerformed

    private void headerBackgroundFocusColoButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_headerBackgroundFocusColoButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("headerBackgroundFocusColor"));
        if (temp != null){
            Config.setColor("headerBackgroundFocusColor", temp);
            headerBackgroundFocusColoButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_headerBackgroundFocusColoButtonActionPerformed

    private void resetButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed
        Config.setColor("colorButtonBackgroundEmpty", Config.getDefaultColor("colorButtonBackgroundEmpty"));
        Config.setColor("colorButtonBackgroundSomething", Config.getDefaultColor("colorButtonBackgroundSomething"));
        Config.setColor("colorButtonBackgroundNothing", Config.getDefaultColor("colorButtonBackgroundNothing"));
        Config.setColor("colorButtonBorderColor", Config.getDefaultColor("colorButtonBorderColor"));
        Config.setColor("headerBackgroundColor", Config.getDefaultColor("headerBackgroundColor"));
        Config.setColor("headerBorderColor", Config.getDefaultColor("headerBorderColor"));
        Config.setColor("headerBackgroundFocusColor", Config.getDefaultColor("headerBackgroundFocusColor"));
        Config.setColor("backgroundColor", Config.getDefaultColor("backgroundColor"));
        Config.setColor("headerFontForeground", Config.getDefaultColor("headerFontForeground"));
        Config.setColor("strikedTextForeground", Config.getDefaultColor("strikedTextForeground"));
        Config.setFont("headerFont", Config.getDefaultFont("headerFont"));
        PreviewWindow.getInstance().pp.setColor(Config.getColor("colorButtonBackgroundSomething"));
//        Config.getInstance().resetToDefaults();
        updateTexts();
        loadColors();
    }//GEN-LAST:event_resetButtonActionPerformed

    private void closeButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        setVisible(false);
    }//GEN-LAST:event_closeButtonActionPerformed

    private void backgroundColorButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_backgroundColorButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("backgroundColor"));
        if (temp != null){
            Config.setColor("backgroundColor", temp);
            backgroundColorButton.setBackground(temp);
            updateTexts();
        }
    }//GEN-LAST:event_backgroundColorButtonActionPerformed

    private void headerFontButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_headerFontButtonActionPerformed
        FontChooser f = new FontChooser((Dialog)this);
        f.setFontAttributesToNormal(false);
        SimpleAttributeSet a = new SimpleAttributeSet();
        StyleConstants.setFontFamily(a, Config.getFont("headerFont").getName());
        StyleConstants.setFontSize(a, Config.getFont("headerFont").getSize());
        StyleConstants.setForeground(a, Config.getColor("headerFontForeground"));
        f.setAttributes(a);
        f.setVisible(true);
        if (f.getOption() == JOptionPane.OK_OPTION){
            Config.setColor("headerFontForeground", f.getForeground());
            Config.setFont("headerFont", f.getFont());
            headerFontButton.setForeground(Config.getColor("headerFontForeground"));
            headerFontButton.setFont(Config.getFont("headerFont"));
            ((MainWindow)getParent()).updateColors();
        }
    }//GEN-LAST:event_headerFontButtonActionPerformed

    private void strikedTextForegroundButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_strikedTextForegroundButtonActionPerformed
        Color temp = JColorChooser.showDialog(this, title, Config.getColor("strikedTextForeground"));
        if (temp != null){
            Config.setColor("strikedTextForeground", temp);
            strikedTextForegroundButton.setBackground(temp);
            ((MainWindow)getParent()).updateColors();
        }
    }//GEN-LAST:event_strikedTextForegroundButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton backgroundButton;
    private javax.swing.JButton backgroundColorButton;
    private javax.swing.JLabel backgroundColorLabel;
    private javax.swing.JButton closeButton;
    private javax.swing.JButton colorButtonBackgroundEmptyButton;
    private javax.swing.JButton colorButtonBackgroundNothingButton;
    private javax.swing.JButton colorButtonBackgroundSomethingButton;
    private javax.swing.JLabel defaultCellLabel;
    private javax.swing.JLabel emptyCellLabel;
    private javax.swing.JButton headerBackgroundColoButton;
    private javax.swing.JButton headerBackgroundFocusColoButton;
    private javax.swing.JButton headerBorderColorButton;
    private javax.swing.JLabel headerBorderLabel;
    private javax.swing.JLabel headerFocusLabel;
    private javax.swing.JButton headerFontButton;
    private javax.swing.JLabel headersLabel;
    private javax.swing.JLabel innerBorderLabel;
    private javax.swing.JLabel markedCellLabel;
    private javax.swing.JButton resetButton;
    private javax.swing.JLabel strikedHeaderColorLabel;
    private javax.swing.JButton strikedTextForegroundButton;
    // End of variables declaration//GEN-END:variables

}
