/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import picto.app.interfaces.Focusable;
import picto.app.interfaces.Playable;
import picto.app.interfaces.Status;
import picto.app.interfaces.Updatable;
import picto.exceptions.IncorrectHeaderException;
import picto.utils.ColoredButton;
import picto.utils.Config;
import picto.utils.DoNothingButton;
import picto.utils.PhraseScrambler;
import picto.utils.Picto;
import picto.utils.PictoSolver;
import picto.utils.Saver;
import picto.utils.TextButton;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class PlayingBoard extends JPanel implements Status, Playable{
    private static final long serialVersionUID = -968907045168963571L;

    private boolean traking_enabled = Config.isMouseTrackingEnabled();
    /**Game over flag, it's used to show the congrats message just once*/
    private boolean game_over = false;
    /**This counts how many cells are correctly marked*/
//    private int cell_counter;
    /**References to the buttons in the game board*/
    private ColoredButton[][] playingBoard;
    /**Current game being played*/
    private PhraseScrambler ps;
    /**Current game being played*/
    private Picto pic;
    /**Number of columns on the screen*/
    private int xSize;
    /**Number of rows on the screen*/
    private int ySize;
    /**Number of column headers*/
    private int ch;
    /**Number of row headers*/
    private int rh;
    /**Number of columns*/
    private int c;
    /**Number of rows*/
    private int r;

    /** Creates new form PlayingBoard */
    public PlayingBoard() {

    }
    
    /** 
     * Creates new form PlayingBoard
     * @param sav 
     */
    public PlayingBoard(Saver sav){
        if (sav.getPs() != null){
            ps = sav.getPs();
            pic = ps.getPic();
        } else {
            pic = sav.getPic();
        }
        setDoubleBuffered(true);
        setBackground(Config.getColor("backgroundColor"));
        regenerate(sav.getStats());
    }

    /** 
     * Creates new form PlayBoard
     * @param pic 
     */
    public PlayingBoard(Picto pic){
        this.pic = pic;
        setDoubleBuffered(true);
        generateButtonGrid();
        setBackground(Config.getColor("backgroundColor"));
    }

    /**
     * Creates new form PlayBoard
     * @param ps 
     */
    public PlayingBoard(PhraseScrambler ps){
        this.pic = ps.getPic();
        this.ps = ps;
        setDoubleBuffered(true);
        generateButtonGrid();
        setBackground(Config.getColor("backgroundColor"));
    }

    /**
     * Generates the button grid
     */
    private void generateButtonGrid(){
        ColoredButton.setContainer(this);
        ColoredButton.setGameOver(false);
        game_over = false;
        ch = pic.getColumnHeaderNum();
        rh = pic.getRowHeaderNum();
        c = pic.getNumberOfColumns();
        r = pic.getNumberOfRows();
        xSize = c + rh;
        ySize = r + ch;
        playingBoard = new ColoredButton[r][c];
        PreviewWindow.getInstance().setNewSize(r, c);
        setLayout(new GridLayout(ySize, xSize, 0, 0));
        byte i, j;
        // Here the buttons are added to the panel, you can accomplish the same
        // with only one loop, but this way is quite more clear
        for (i = 0; i < ch; i++){
            // Upper left unused buttons
            for (j = 0; j < rh; j++){
                add(new DoNothingButton());
            }
            // Column headers
            for (j = 0; j < c; j++){
                add(new TextButton(pic.getColumnHeadersElement(j, i), true, j));
            }
        }
        for (i = 0; i < r; i++){
            // Row headers
            for (j = 0; j < rh; j++){
                add(new TextButton(pic.getRowHeadersElement(i, j), false, i));
            }
            // Playable cells
            if (ps != null){// Phrase game
                for (j = 0; j < c; j++){
                    playingBoard[i][j] = new ColoredButton(i, j);
                    playingBoard[i][j].setText(String.valueOf(ps.getElement(i, j)));
                    add(playingBoard[i][j]);
                }
            } else {// Regular game
                for (j = 0; j < c; j++){
                    playingBoard[i][j] = new ColoredButton(i, j);
                    add(playingBoard[i][j]);
                }
            }
        }
        updateUI();
    }

    /**
     * Turns tracking on and off
     *
     * @param state
     */
    public void setTrakingEnabled(boolean state){
        traking_enabled = state;
        // If you turn tracking off you need to clear the headers
        if (!traking_enabled){
            int _temp, i;
            _temp = old_j_focus + rh;
            for (i = ch-1; i >= 0 ; i--){
                ((Focusable)getComponent(i*xSize+_temp)).removeFocus();
            }
            _temp = (ch+old_i_focus)*xSize+rh;
            for (i = rh-1; i >= 0; i--){
                ((Focusable)getComponent(--_temp)).removeFocus();
            }
            old_i_focus = 0;
            old_j_focus = 0;
        }
    }

    private int old_i_focus = 0, old_j_focus = 0;

    /**
     * If tracking is enabled, it changes the color of the header in the
     * same line and row of the mouse position
     * @FIXME this is awfully slow
     * @param i
     * @param j
     */
    @Override public void updateHeaderFocus(int i, int j){
        if (!traking_enabled){
            return;
        }
        int g_temp , r_temp, k;
        if (old_j_focus != j){
            g_temp = j + rh;
            r_temp = old_j_focus + rh;
            for (k = ch-1; k >= 0 ; k--){
                ((Focusable)getComponent(k*xSize+r_temp)).removeFocus();
                ((Focusable)getComponent(k*xSize+g_temp)).giveFocus();
            }
            old_j_focus = j;
        }
        if (old_i_focus != i){
            g_temp = (ch+i)*xSize+rh;
            r_temp = (ch+old_i_focus)*xSize+rh;
            for (k = rh-1; k >= 0; k--){
                ((Focusable)getComponent(--r_temp)).removeFocus();
                ((Focusable)getComponent(--g_temp)).giveFocus();
            }
            old_i_focus = i;
        }
    }


    @Override public int getXSize() {
        return xSize;
    }

    @Override public int getYSize() {
        return ySize;
    }

    @Override public void solve(){
        for (int i = 0; i < r; i++){
            for (int j = 0; j < c; j++){
                if (pic.getElement(i, j)){
                    playingBoard[i][j].setStatus(FULL);
                } else {
                    playingBoard[i][j].setStatus(EMPTY);
                }
            }
        }
        if (ps != null){
            String text = "";
            String [] temp = ps.getPhrase().split("--");
            text += temp[0]+"\n";
            if (temp.length == 2){
                text += temp[1];
            }
            JOptionPane.showMessageDialog(this, text
                , _("Cheater!"), JOptionPane.INFORMATION_MESSAGE);
        } 
    }

    @Override public void clean(){
        for (Component but : getComponents()){
            if (but instanceof Updatable){
                ((Updatable)but).reset();
            }
        }
    }

    @Override public Picto getPic() {
        return pic;
    }

    /**
     * Restore the previous status of a board, this is used when loading a game
     *
     * @param stats the stats contained in the Saver object
     * @see getSaver()
     */
    private void regenerate(byte[] stats){
        pic.regenerate();
        generateButtonGrid();
        int counter = 0;
        for (Component but : getComponents()){
            ((Updatable)but).setStatus(stats[counter++]);
        }
        stats = null;
    }

    /**
     * Updates the colors in case of a change in the ColorChooser
     * @see picto.app.gui.ColorChooser
     */
    @Override public void updateColors(){
        for (Component but : getComponents()){
            ((Updatable)but).update();
        }
        setBackground(Config.getColor("backgroundColor"));
    }

    /**
     * Update all the headers when the font is changed
     */
    public void updateHeaderFont(){
        for (Component but : getComponents()){
            if (but instanceof TextButton){
                ((TextButton)but).update();
            }
        }
    }

    @Override public void checkBoard(boolean reset_wrong){
        int false_positive = 0; // Elements that shouldn't be marked as full
        int false_negative = 0; // Elements that shouldn't be marked as empty
        int counter = pic.getNumberOfMarkedCells(); // Number of marked cells
        for (int i = 0; i < r; i++){
            for (int j = 0; j < c; j++){
                if (playingBoard[i][j].getStatus() == FULL){
                    if (!pic.getElement(i, j)){
                        if (reset_wrong){
                            playingBoard[i][j].reset();
                        }
                        false_positive++;
                    } else {
                        counter--;
                    }
                } else if (playingBoard[i][j].getStatus() == EMPTY
                        && pic.getElement(i, j)){
                    if (reset_wrong){
                        playingBoard[i][j].reset();
                    }
                    false_negative++;
                }
            }
        }
        // What to do if the if everything is A_OK
        if ((counter|false_negative|false_positive)==0){
            String text = "";
            if (ps != null){ // If the panel is a phrase show it.
                String [] temp = ps.getPhrase().split("--");
                text += temp[0];
                if (temp.length == 2){
                    text += "\n"+temp[1];
                }
            } else {
                text = _("Congratulations!");
            }
            JOptionPane.showMessageDialog(this, text
                    , _("Cool!"), JOptionPane.INFORMATION_MESSAGE);
        } else {// What to do if everything is ok but the game isn't finished
            if ((false_positive|false_negative) == 0){
                String text = _("The game is not finished yet.") + "\n"
                            + _("No elements where wrongly marked");
                JOptionPane.showMessageDialog(this, text, _("Cool!")
                                  , JOptionPane.INFORMATION_MESSAGE);
            } else {// What to do in any other case
                String text = false_positive + " "
                            + _("cells shouldn't be marked as full") + "\n"
                            + false_negative + " "
                            + _("cells shouldn't be marked as empty");
                JOptionPane.showMessageDialog(this, text, _("Oops!")
                        , JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * Saves the current state of all the board and returns a saver
     * @return Saver object
     * @see picto.utils.Saver
     */
    public Saver getSaver(){
        Saver temp = new Saver();
        if (ps != null){
            temp.setPs(ps);
        } else {
            temp.setPic(pic);
        }
        byte[] stats = new byte[xSize * ySize];
        int counter = 0;
        for (Component but : getComponents()){
            stats[counter++] = ((Updatable)but).getStatus();
        }
        temp.setStats(stats);
        return temp;
    }

    /**
     * This method checks if you won a game... is very crude but it's a nice
     * feature...
     * @TODO I implemented a nice method with a counter once, but it had to
     * many problems, so I'll leave this one for now... I promise I will fix
     * it eventually...
     */
    public void checkBoard(){
        if (game_over){
            return;
        }
        
        for (byte i = 0; i < r; i++){
            for (byte j = 0; j < c; j++){
                if (pic.getElement(i, j)){
                    if (playingBoard[i][j].getStatus() != Status.FULL){
                        return;
                    }
                } else {
                    if (playingBoard[i][j].getStatus() == Status.FULL){
                        return;
                    }
                }
            }
        }
        youWon();
    }

    public void youWon(){
        ColoredButton.setGameOver(true);
        checkBoard(false);
        game_over = true;
    }

    public void attemptSolving(){
        try {
            clean();
            PictoSolver solver = new PictoSolver(pic);
            solver.setButtonBoard(playingBoard);
            solver.solve();
            solver.flush();
        } catch (IncorrectHeaderException ex) {
//            Logger.getLogger(PlayingBoard.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void printPanel(){
        PrinterJob job = PrinterJob.getPrinterJob();
        // set the Printable to the PrinterJob
        job.setPrintable(new Printable() {
            @Override public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
                if (pageIndex == 0) {
                    Graphics2D g2d = (Graphics2D)graphics;
                    g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
                    PlayingBoard.this.paint(g2d);
                    PlayingBoard.this.printAll(g2d);
                    return Printable.PAGE_EXISTS;
                } else {
                    return Printable.NO_SUCH_PAGE;
                }
            }
        });

        // show the dialog
        if (job.printDialog()) {
            try {
                job.print();
            }
            catch (PrinterException ex) {
            // handle exception
            }
        }
    }
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 154, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 136, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
