/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.interfaces;

import picto.utils.Picto;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public interface Playable {

    /**
     * Gets the current number of columns
     * @return current number of columns on the screen
     */
    int getXSize();

    /**
     * Gets the current number of rows
     * @return current number of rows on the screen
     */
    int getYSize();

    /**
     * Cleans the board
     */
    void clean();

    /**
     * Shows the solution on the screen (It's not necessary to calculate it,
     * just show it)
     */
    void solve();

    /**
     * Updates the colors of the button, that is, background, borders, text,
     * etc.
     */
    void updateColors();

    /**
     * Checks the board, and displays a message according to it's status
     *
     * @param reset_wrong this flag indicates if it's necessary to clean the
     * wrongly marked cells
     */
    void checkBoard(boolean reset_wrong);

    /**
     * Updates the status of the focus in a header
     *
     * @param i
     * @param j
     */
    void updateHeaderFocus(int i, int j);

    /**
     * @return the Picto of the current game
     */
    Picto getPic();
}
